@@ -47,6 +47,14 @@ void KConfigHeaderGenerator::doClassDefinition()
     if (!parseResult.signalList.isEmpty() || cfg().generateProperties) {
         stream() << "  Q_OBJECT\n";
     }
+
+    if (cfg().qmlRegistration) {
+        stream() << "  QML_ELEMENT\n";
+
+        if (cfg().singleton) {
+            stream() << "  QML_SINGLETON\n";
+        }
+    }
     stream() << "  public:\n";
     implementEnums();
     createConstructor();
@@ -114,6 +122,14 @@ void KConfigHeaderGenerator::createHeaders()
     }
 
     addHeaders({QStringLiteral("QCoreApplication"), QStringLiteral("QDebug")});
+    if (!cfg().dpointer && parseResult.hasNonModifySignals) {
+        addHeaders({QStringLiteral("QSet")});
+    }
+
+    if (cfg().qmlRegistration) {
+        addHeaders({QStringLiteral("qqmlintegration.h")});
+    }
+
     stream() << '\n';
 
     addHeaders(parseResult.includes);
@@ -217,20 +233,11 @@ void KConfigHeaderGenerator::implementEnums()
 void KConfigHeaderGenerator::createSignals()
 {
     // Signal definition.
-    const bool hasSignals = !parseResult.signalList.isEmpty();
-
-    unsigned val = 1 << parseResult.signalList.size();
-    if (!val) {
-        std::cerr << "Too many signals to create unique bit masks" << std::endl;
-        exit(1);
-    }
-
-    if (!hasSignals) {
+    if (parseResult.signalList.isEmpty()) {
         return;
     }
 
     stream() << "\n    enum {\n";
-    val = 1;
 
     // HACK: Use C-Style for add a comma in all but the last element,
     // just to make the source generated code equal to the old one.
@@ -238,15 +245,13 @@ void KConfigHeaderGenerator::createSignals()
     // a last comma, as it's valid c++.
     for (int i = 0, end = parseResult.signalList.size(); i < end; i++) {
         auto signal = parseResult.signalList.at(i);
-        stream() << whitespace() << "  " << signalEnumName(signal.name) << " = 0x" << Qt::hex << val;
+        stream() << whitespace() << "  " << signalEnumName(signal.name) << " = " << (i + 1);
         if (i != end - 1) {
             stream() << ",\n";
         }
-
-        val <<= 1;
     }
     stream() << '\n';
-    stream() << whitespace() << "};" << Qt::dec << "\n\n";
+    stream() << whitespace() << "};\n\n";
 
     stream() << "  Q_SIGNALS:";
     for (const Signal &signal : std::as_const(parseResult.signalList)) {
@@ -281,7 +286,7 @@ void KConfigHeaderGenerator::createSignals()
     stream() << '\n';
 
     stream() << "  private:\n";
-    stream() << whitespace() << "void itemChanged(quint64 flags);\n";
+    stream() << whitespace() << "void itemChanged(quint64 signalFlag);\n";
     stream() << '\n';
 }
 
@@ -313,6 +318,11 @@ void KConfigHeaderGenerator::createConstructor()
 {
     if (cfg().singleton) {
         stream() << whitespace() << "static " << cfg().className << " *self();\n";
+
+        if (cfg().qmlRegistration) {
+            stream() << whitespace() << "static " << cfg().className << " *create(QQmlEngine *, QJSEngine *);\n";
+        }
+
         if (parseResult.cfgFileNameArg) {
             stream() << whitespace() << "static void instance(const QString& cfgfilename);\n";
             stream() << whitespace() << "static void instance(KSharedConfig::Ptr config);\n";
@@ -367,6 +377,11 @@ void KConfigHeaderGenerator::createForwardDeclarations()
     if (cfg().dpointer) {
         stream() << "class " << cfg().className << "Private;\n\n";
     }
+
+    if (cfg().qmlRegistration && cfg().singleton) {
+        stream() << "class QQmlEngine;\n";
+        stream() << "class QJSEngine;\n\n";
+    }
 }
 
 void KConfigHeaderGenerator::createProperties(const CfgEntry *entry, const QString &returnType)
@@ -639,6 +654,6 @@ void KConfigHeaderGenerator::createNonDPointerHelpers()
     }
 
     if (parseResult.hasNonModifySignals) {
-        stream() << whitespace() << "uint " << varName(QStringLiteral("settingsChanged"), cfg()) << ";\n";
+        stream() << whitespace() << "QSet<quint64> " << varName(QStringLiteral("settingsChanged"), cfg()) << ";\n";
     }
 }
